@@ -14,7 +14,7 @@ static unsigned char * DirWithThumbnailPtrs;
 static double FocalplaneXRes;
 static double FocalplaneUnits;
 static int ExifImageWidth;
-int MotorolaOrder = 0;
+static int MotorolaOrder = 0;
 
 // for fixing the rotation.
 static void * OrientationPtr[2];
@@ -143,7 +143,6 @@ const int BytesPerFormat[] = {0,1,1,2,4,8,1,1,2,4,8,4,8};
 #define TAG_LIGHT_SOURCE           0x9208
 #define TAG_FLASH                  0x9209
 #define TAG_FOCALLENGTH            0x920A
-#define TAG_SUBJECTAREA            0x9214
 #define TAG_MAKER_NOTE             0x927C
 #define TAG_USERCOMMENT            0x9286
 #define TAG_SUBSEC_TIME            0x9290
@@ -158,8 +157,8 @@ const int BytesPerFormat[] = {0,1,1,2,4,8,1,1,2,4,8,4,8};
 
 #define TAG_FLASH_PIX_VERSION      0xA000
 #define TAG_COLOR_SPACE            0xA001
-#define TAG_PIXEL_X_DIMENSION      0xA002
-#define TAG_PIXEL_Y_DIMENSION      0xA003
+#define TAG_EXIF_IMAGEWIDTH        0xA002
+#define TAG_EXIF_IMAGELENGTH       0xA003
 #define TAG_RELATED_AUDIO_FILE     0xA004
 #define TAG_INTEROP_OFFSET         0xA005
 #define TAG_FLASH_ENERGY           0xA20B
@@ -184,7 +183,6 @@ const int BytesPerFormat[] = {0,1,1,2,4,8,1,1,2,4,8,4,8};
 #define TAG_SATURATION             0xA409
 #define TAG_SHARPNESS              0xA40A
 #define TAG_DISTANCE_RANGE         0xA40C
-#define TAG_IMAGE_UNIQUE_ID        0xA420
 
 static const TagTable_t TagTable[] = {
   { TAG_INTEROP_INDEX,          "InteropIndex"},
@@ -265,8 +263,8 @@ static const TagTable_t TagTable[] = {
   { TAG_WINXP_SUBJECT,          "Windows-XP subject"},
   { TAG_FLASH_PIX_VERSION,      "FlashPixVersion"},
   { TAG_COLOR_SPACE,            "ColorSpace"},
-  { TAG_PIXEL_X_DIMENSION,      "ExifImageWidth"},
-  { TAG_PIXEL_Y_DIMENSION,      "ExifImageLength"},
+  { TAG_EXIF_IMAGEWIDTH,        "ExifImageWidth"},
+  { TAG_EXIF_IMAGELENGTH,       "ExifImageLength"},
   { TAG_RELATED_AUDIO_FILE,     "RelatedAudioFile"},
   { TAG_INTEROP_OFFSET,         "InteroperabilityOffset"},
   { TAG_FLASH_ENERGY,           "FlashEnergy"},              
@@ -285,14 +283,12 @@ static const TagTable_t TagTable[] = {
   { TAG_WHITEBALANCE,           "WhiteBalance"},
   { TAG_DIGITALZOOMRATIO,       "DigitalZoomRatio"},
   { TAG_FOCALLENGTH_35MM,       "FocalLengthIn35mmFilm"},
-  { TAG_SUBJECTAREA,            "SubjectArea"},
   { TAG_SCENE_CAPTURE_TYPE,     "SceneCaptureType"},
   { TAG_GAIN_CONTROL,           "GainControl"},
   { TAG_CONTRAST,               "Contrast"},
   { TAG_SATURATION,             "Saturation"},
   { TAG_SHARPNESS,              "Sharpness"},
   { TAG_DISTANCE_RANGE,         "SubjectDistanceRange"},
-  { TAG_IMAGE_UNIQUE_ID,        "ImageUniqueId"},
 } ;
 
 #define TAG_TABLE_SIZE  (sizeof(TagTable) / sizeof(TagTable_t))
@@ -380,14 +376,10 @@ void PrintFormatNumber(void * ValuePtr, int Format, int ByteCount)
             case FMT_SLONG:     printf("%d",Get32s(ValuePtr)); s=4;      break;
             case FMT_SSHORT:    printf("%hd",(signed short)Get16u(ValuePtr)); s=2; break;
             case FMT_URATIONAL:
-                printf("%u/%u",Get32s(ValuePtr), Get32s(4+(char *)ValuePtr)); 
-                s = 8;
-                break;
-
             case FMT_SRATIONAL: 
-                printf("%d/%d",Get32s(ValuePtr), Get32s(4+(char *)ValuePtr)); 
-                s = 8;
-                break;
+               printf("%d/%d",Get32s(ValuePtr), Get32s(4+(char *)ValuePtr)); 
+               s = 8;
+               break;
 
             case FMT_SINGLE:    printf("%f",(double)*(float *)ValuePtr); s=8; break;
             case FMT_DOUBLE:    printf("%f",*(double *)ValuePtr);        s=8; break;
@@ -429,11 +421,7 @@ double ConvertAnyFormat(void * ValuePtr, int Format)
                 if (Den == 0){
                     Value = 0;
                 }else{
-                    if (Format == FMT_SRATIONAL){
-                        Value = (double)Num/Den;
-                    }else{
-                        Value = (double)(unsigned)Num/(double)(unsigned)Den;
-                    }
+                    Value = (double)Num/Den;
                 }
                 break;
             }
@@ -558,11 +546,13 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
             // Show tag name
             for (a=0;;a++){
                 if (a >= TAG_TABLE_SIZE){
-                    printf("%s    Unknown Tag %04x Value = ", IndentString, Tag);
+                    printf(IndentString);
+                    printf("    Unknown Tag %04x Value = ", Tag);
                     break;
                 }
                 if (TagTable[a].Tag == Tag){
-                    printf("%s    %s = ",IndentString, TagTable[a].Desc);
+                    printf(IndentString);
+                    printf("    %s = ",TagTable[a].Desc);
                     break;
                 }
             }
@@ -652,7 +642,7 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                 if (ByteCount > 1){
                     if (ByteCount > MAX_COMMENT_SIZE) ByteCount = MAX_COMMENT_SIZE;
                     memcpy(ImageInfo.Comments, ValuePtr, ByteCount);
-                    ImageInfo.CommentWidthchars = ByteCount/2;
+                    ImageInfo.CommentWidchars = ByteCount/2;
                 }
                 break;
 
@@ -675,21 +665,17 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                 }
 
                 // Copy the comment
-                {
-                    int msiz = ExifLength - (ValuePtr-OffsetBase);
-                    if (msiz > ByteCount) msiz = ByteCount;
-                    if (msiz > MAX_COMMENT_SIZE-1) msiz = MAX_COMMENT_SIZE-1;
-                    if (msiz > 5 && memcmp(ValuePtr, "ASCII",5) == 0){
-                        for (a=5;a<10 && a < msiz;a++){
-                            int c = (ValuePtr)[a];
-                            if (c != '\0' && c != ' '){
-                                strncpy(ImageInfo.Comments, (char *)ValuePtr+a, msiz-a);
-                                break;
-                            }
+                if (memcmp(ValuePtr, "ASCII",5) == 0){
+                    for (a=5;a<10;a++){
+                        int c;
+                        c = (ValuePtr)[a];
+                        if (c != '\0' && c != ' '){
+                            strncpy(ImageInfo.Comments, (char *)ValuePtr+a, 199);
+                            break;
                         }
-                    }else{
-                        strncpy(ImageInfo.Comments, (char *)ValuePtr, msiz);
                     }
+                }else{
+                    strncpy(ImageInfo.Comments, (char *)ValuePtr, MAX_COMMENT_SIZE-1);
                 }
                 break;
 
@@ -760,8 +746,8 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                 NumOrientations += 1;
                 break;
 
-            case TAG_PIXEL_Y_DIMENSION:
-            case TAG_PIXEL_X_DIMENSION:
+            case TAG_EXIF_IMAGELENGTH:
+            case TAG_EXIF_IMAGEWIDTH:
                 // Use largest of height and width to deal with images that have been
                 // rotated to portrait format.
                 a = (int)ConvertAnyFormat(ValuePtr, Format);
@@ -823,6 +809,10 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
 
             case TAG_ISO_EQUIVALENT:
                 ImageInfo.ISOequivalent = (int)ConvertAnyFormat(ValuePtr, Format);
+                if ( ImageInfo.ISOequivalent < 50 ){
+                    // Fixes strange encoding on some older digicams.
+                    ImageInfo.ISOequivalent *= 200;
+                }
                 break;
 
             case TAG_DIGITALZOOMRATIO:
@@ -864,7 +854,7 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                     if (SubdirStart < OffsetBase || SubdirStart > OffsetBase+ExifLength){
                         ErrNonfatal("Illegal GPS directory link in Exif",0,0);
                     }else{
-                        ProcessGpsInfo(SubdirStart, OffsetBase, ExifLength);
+                        ProcessGpsInfo(SubdirStart, ByteCount, OffsetBase, ExifLength);
                     }
                     continue;
                 }
@@ -882,31 +872,6 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
                 //   1 = macro, 2 = close, 3 = distant
                 ImageInfo.DistanceRange = (int)ConvertAnyFormat(ValuePtr, Format);
                 break;
-
-
-
-            case TAG_X_RESOLUTION:
-                if (NestingLevel==0) {// Only use the values from the top level directory
-                    ImageInfo.xResolution = (float)ConvertAnyFormat(ValuePtr,Format);
-                    // if yResolution has not been set, use the value of xResolution
-                    if (ImageInfo.yResolution == 0.0) ImageInfo.yResolution = ImageInfo.xResolution;
-                }
-                break;
-
-            case TAG_Y_RESOLUTION:
-                if (NestingLevel==0) {// Only use the values from the top level directory
-                    ImageInfo.yResolution = (float)ConvertAnyFormat(ValuePtr,Format);
-                    // if xResolution has not been set, use the value of yResolution
-                    if (ImageInfo.xResolution == 0.0) ImageInfo.xResolution = ImageInfo.yResolution;
-                }
-                break;
-
-            case TAG_RESOLUTION_UNIT:
-                if (NestingLevel==0) {// Only use the values from the top level directory
-                    ImageInfo.ResolutionUnit = (int) ConvertAnyFormat(ValuePtr,Format);
-                }
-                break;
-
         }
     }
 
@@ -980,7 +945,7 @@ static void ProcessExifDir(unsigned char * DirStart, unsigned char * OffsetBase,
 //--------------------------------------------------------------------------
 void process_EXIF (unsigned char * ExifSection, unsigned int length)
 {
-    unsigned int FirstOffset;
+    int FirstOffset;
 
     FocalplaneXRes = 0;
     FocalplaneUnits = 0;
@@ -1020,10 +985,6 @@ void process_EXIF (unsigned char * ExifSection, unsigned int length)
 
     FirstOffset = Get32u(ExifSection+12);
     if (FirstOffset < 8 || FirstOffset > 16){
-        if (FirstOffset < 16 || FirstOffset > length-16){
-            ErrNonfatal("invalid offset for first Exif IFD value",0,0);
-            return;
-        }
         // Usually set to 8, but other values valid too.
         ErrNonfatal("Suspicious offset of first Exif IFD value",0,0);
     }
@@ -1048,7 +1009,7 @@ void process_EXIF (unsigned char * ExifSection, unsigned int length)
 
 
     // Compute the CCD width, in millimeters.
-    if (FocalplaneXRes != 0 && ExifImageWidth != 0){
+    if (FocalplaneXRes != 0){
         // Note: With some cameras, its not possible to compute this correctly because
         // they don't adjust the indicated focal plane resolution units when using less
         // than maximum resolution, so the CCDWidth value comes out too small.  Nothing
@@ -1224,7 +1185,7 @@ const char * ClearOrientation(void)
 
             case FMT_ULONG:     
             case FMT_SLONG:     
-                memset(OrientationPtr[a], 0, 4);
+                memset(OrientationPtr, 0, 4);
                 // Can't be bothered to write  generic Put32 if I only use it once.
                 if (MotorolaOrder){
                     ((uchar *)OrientationPtr[a])[3] = 1;
@@ -1241,6 +1202,52 @@ const char * ClearOrientation(void)
     return OrientTab[ImageInfo.Orientation];
 }
 
+
+
+//--------------------------------------------------------------------------
+// Remove thumbnail out of the exif image.
+//--------------------------------------------------------------------------
+int RemoveThumbnail(unsigned char * ExifSection)
+{
+    if (!DirWithThumbnailPtrs || 
+        ImageInfo.ThumbnailOffset == 0 || 
+        ImageInfo.ThumbnailSize == 0){
+        // No thumbnail, or already deleted it.
+        return 0;
+    }
+    if (ImageInfo.ThumbnailAtEnd == FALSE){
+        ErrNonfatal("Thumbnail not at end of Exif header, can't remove it", 0, 0);
+        return 0;
+    }
+
+    {
+        int de;
+        int NumDirEntries;
+        NumDirEntries = Get16u(DirWithThumbnailPtrs);
+
+        for (de=0;de<NumDirEntries;de++){
+            int Tag;
+            unsigned char * DirEntry;
+            DirEntry = DIR_ENTRY_ADDR(DirWithThumbnailPtrs, de);
+            Tag = Get16u(DirEntry);
+            if (Tag == TAG_THUMBNAIL_LENGTH){
+                // Set length to zero.
+                if (Get16u(DirEntry+2) != FMT_ULONG){
+                    // non standard format encoding.  Can't do it.
+                    ErrNonfatal("Can't remove Exif thumbnail", 0, 0);
+                    return 0;
+                }
+                Put32u(DirEntry+8, 0);
+            }                    
+        }
+    }
+
+    // This is how far the non thumbnail data went.
+    return ImageInfo.ThumbnailOffset+8;
+
+}
+
+
 //--------------------------------------------------------------------------
 // Convert exif time to Unix time structure
 //--------------------------------------------------------------------------
@@ -1259,17 +1266,6 @@ int Exif2tm(struct tm * timeptr, char * ExifTime)
             &timeptr->tm_hour, &timeptr->tm_min, &timeptr->tm_sec);
 
     if (a >= 5){
-        if (timeptr->tm_year <= 12 && timeptr->tm_mday > 2000 && ExifTime[2] == '.'){
-            // LG Electronics VX-9700 seems to encode the date as 'MM.DD.YYYY HH:MM'
-            // can't these people read the standard?  At least they left enough room
-            // in the header to put an Exif format date in there.
-            int tmp;
-            tmp = timeptr->tm_year;
-            timeptr->tm_year = timeptr->tm_mday;
-            timeptr->tm_mday = timeptr->tm_mon;
-            timeptr->tm_mon = tmp;
-        }
-
         // Accept five or six parameters.  Some cameras do not store seconds.
         timeptr->tm_isdst = -1;  
         timeptr->tm_mon -= 1;      // Adjust for unix zero-based months 
@@ -1431,17 +1427,17 @@ void ShowImageInfo(int ShowFileInfo)
             // don't bother showing it - it doesn't add any useful information.
     }
 
-    if (ImageInfo.MeteringMode > 0){ // 05-jan-2001 vcs
-        printf("Metering Mode: ");
+    if (ImageInfo.MeteringMode){ // 05-jan-2001 vcs
         switch(ImageInfo.MeteringMode) {
-        case 1: printf("average\n"); break;
-        case 2: printf("center weight\n"); break;
-        case 3: printf("spot\n"); break;
-        case 4: printf("multi spot\n");  break;
-        case 5: printf("pattern\n"); break;
-        case 6: printf("partial\n");  break;
-        case 255: printf("other\n");  break;
-        default: printf("unknown (%d)\n",ImageInfo.MeteringMode); break;
+        case 2:
+            printf("Metering Mode: center weight\n");
+            break;
+        case 3:
+            printf("Metering Mode: spot\n");
+            break;
+        case 5:
+            printf("Metering Mode: matrix\n");
+            break;
         }
     }
 
@@ -1505,7 +1501,7 @@ void ShowImageInfo(int ShowFileInfo)
     if (ImageInfo.Process != M_SOF0){
         // don't show it if its the plain old boring 'baseline' process, but do
         // show it if its something else, like 'progressive' (used on web sometimes)
-        unsigned a;
+        int a;
         for (a=0;;a++){
             if (a >= PROCESS_TABLE_SIZE){
                 // ran off the end of the table.
@@ -1525,15 +1521,11 @@ void ShowImageInfo(int ShowFileInfo)
         if (ImageInfo.GpsAlt[0]) printf("GPS Altitude : %s\n",ImageInfo.GpsAlt);
     }
 
-    if (ImageInfo.QualityGuess){
-        printf("JPEG Quality : %d\n", ImageInfo.QualityGuess);
-    }
-
     // Print the comment. Print 'Comment:' for each new line of comment.
     if (ImageInfo.Comments[0]){
         int a,c;
         printf("Comment      : ");
-        if (!ImageInfo.CommentWidthchars){
+        if (!ImageInfo.CommentWidchars){
             for (a=0;a<MAX_COMMENT_SIZE;a++){
                 c = ImageInfo.Comments[a];
                 if (c == '\0') break;
@@ -1550,7 +1542,7 @@ void ShowImageInfo(int ShowFileInfo)
             }
             printf("\n");
         }else{
-            printf("%.*ls\n", ImageInfo.CommentWidthchars, (wchar_t *)ImageInfo.Comments);
+            printf("%.*ls\n", ImageInfo.CommentWidchars, (wchar_t *)ImageInfo.Comments);
         }
     }
 }
